// Program SER_87X.C
//
// This is a collection of routines to interface with a PIC-an-LCD
// or similar serial LCD capable of 9600 baud, noninverted, no parity.
//
// It also includes routines to receive characters and character strings
// on RC.7 (term 26).  9600 baud, noninverted, no parity.
//
//  16F877   			 Serial LCD+
//
// RC6 (term 25) -----------------> Serin In
//
//
// RC7 (term 26) <---------------- TX (term 12 on BX24)
//
// copyright, Peter H. Anderson, Baltimore, MD, Georgetown, SC, Mar, '01
// Modified by Arnan (Roger) Sipitakiat, MIT Media Laboratory
//

byte rx_buff[RX_BUFF_MAX];
byte rx_put_index, rx_get_index, rx_buff_full;
byte x;


void ser_init(void) // sets TxData in idle state
{
   asynch_enable();
   open_rx_com();
}

void asynch_enable(void)
{
   trisc6 = 1; // make tx (term 25 an input)
   trisc7 = 1; // rx (term 26) input

   sync = 0;   // asynchronous
   brgh = 1 ;  // baud rate generator high speed
//   SPBRG = 25;  // 9600 with 4.0 MHz clock
	SPBRG = 129;
//   SPBRG = 129 for 9600 baud with 20.0 MHz clock

   spen = 1;    // serial port enabled

   txen = 1;  // as appropriate
   cren = 1;
}

void asynch_disable(void)
{
   txen = 0;
   cren = 0;
   spen = 0;
}

void ser_char(char ch) // no interrupts
{
   byte n = 250;

   delay_ms(1);		//  required for Serial LCD+
   while(!txif && --n)	/* txif goes to one when buffer is empty */
   {
	   delay_us(10);
   }
   TXREG = ch;    	//  This clears txif
}


void ser_out_str(char *s)
{
	byte n = 0;

    while ((s[n] != 0) && (n<20))
    {
       ser_char(s[n]);
       ++n;
    }
}

char ser_get_ch(long t_wait)
// returns 0xff if no char received within t_wait ms
{
   byte loop_one_ms, n;
   char ch;

   rcif=0;

   do
   {
      loop_one_ms = 100;
      do
      {			// assumed to be 10 instruction cycles
#asm			// check and adjust with NOPs as neccessary
	CLRWDT
#endasm
         if(rcif)
         {
            return(RCREG);
         }
      } while(loop_one_ms--);
   } while(t_wait--);
   return(0xff);	// if timeout
}



void ser_hex_byte(byte val) // displays val in hex format
{
   char ch;
   ch = num_to_char((val>>4) & 0x0f);
   ser_char(ch);
   ch = num_to_char(val&0x0f);
   ser_char(ch);
}

void ser_dec_byte(byte val, byte digits)
// displays byte in decimal as either 1, 2 or 3 digits
{
   byte d;
   char ch;
   if (digits == 3)
   {
      d=val/100;
      ch=num_to_char(d);
      ser_char(ch);
   }
   if (digits >1) // take the two lowest digits
   {
       val=val%100;
       d=val/10;
       ch=num_to_char(d);
       ser_char(ch);
   }
   if (digits == 1) // take the least significant digit
   {
       val = val%100;
   }

   d=val % 10;
   ch=num_to_char(d);
   ser_char(ch);
}


//////////////////////////////////////////////////////////////////////////////



char num_to_char(byte val) // converts val to hex character
{
   char ch;
   if (val < 10)
   {
     ch=val+'0';
   }
   else
   {
     val=val-10;
     ch=val + 'A';
   }
   return(ch);
}



void open_rx_com(void)
{
    char ch;
    asynch_enable();
	rx_put_index = 0;
	rx_get_index = 0;
	rx_buff_full = FALSE;
	ch = RCREG;		// get any junk that may be in the buffer
	ch = RCREG;

	rcif = 0;
    rcie = 1;   // enable rda interrupt
    peie = 1;	// enable periperal interrupts
	gie = 1;	// Global interrup enable
}


/////////////////////////////////////////////////////////////////////////
//
//   I N T E R R U P T    S E R V I C E    R O U T I N E S
//
/////////////////////////////////////////////////////////////////////////






#int_rda
rda_interrupt_handler(void)
{

   //x = x;  // what is this for??
   if (rx_buff_full == FALSE)
   {
      rx_buff[rx_put_index] = RCREG;	// fetch the character
  	  ++rx_put_index;
      if (rx_put_index >= RX_BUFF_MAX)
	  {
	     rx_put_index = 0;   // wrap around
      }
	  if (rx_put_index == rx_get_index)
	  {
	     rx_buff_full = TRUE;
	  }
   }
}




#int_default
default_interrupt_handler(void)
{
}

////////////////////////////////////////////////////////////////////////////


byte get_rx_buff(byte *ch_ptr)
{
   byte error_code;
   if (rx_buff_full == TRUE)
   {
	   rx_buff_full = FALSE;
	   error_code = OVERFLOW;	// overflow of rx_buff
  	   rx_put_index = 0;
  	   rx_get_index = 0;
	   *ch_ptr = 0;		// buff was full.  returned character has no meaning
	}
	else if (rx_get_index == rx_put_index)	// there is no character
	{
	    error_code = NO_DATA;		// no character
		*ch_ptr = 0;
	}
	else
	{
	    *ch_ptr = rx_buff[rx_get_index];
		++rx_get_index;
		if (rx_get_index >= RX_BUFF_MAX)
		{
		   rx_get_index = 0;
        }
		error_code = SUCCESS;	// success
	}
	return(error_code);
}
